import pandas as pd
import numpy as np
from datetime import datetime, timezone
import requests
import json
import time
import os

def _instacart_poll_report_status(report_id, access_token):
    url = f"https://api.ads.instacart.com/api/v2/reports/{report_id}"
    headers = {
        'Accept': 'application/json',
        'Authorization': f'Bearer {access_token}',
    }
    start_time = time.time()
    timeout_seconds = 3600  # 1 hour

    while True:
        if time.time() - start_time > timeout_seconds:
            raise TimeoutError("Instacart report processing timed out after 1 hour.")

        response = requests.get(url, headers=headers)
        if response.status_code != 200:
            print(f"Failed to get Instacart report status: {response.status_code} - {response.text}")
            time.sleep(60)
            continue

        status = response.json()['data']['attributes']['status']
        print(f'Report Status: {status}')

        if status == 'completed':
            return True
        elif status in ('failed', 'cancelled'):
            raise ValueError(f"Instacart report generation failed with status: {status}")
        else:
            time.sleep(60)

def insta_api_get_refresh_token(client_id, client_secret, code):
    token_url = "https://api.ads.instacart.com/oauth/token"

    headers = {
        "Content-Type": "application/json"
    }

    data = {
        "client_id": client_id,
        "client_secret": client_secret,
        "redirect_uri": 'https://zvdataautomation.com/',
        "code": code,
        "grant_type": 'authorization_code',
    }

    response = requests.post(
        token_url,
        headers=headers,
        json=data,
    )

    try:
      refresh_token = response.json()['refresh_token']
      return refresh_token

    except requests.exceptions.RequestException as e:
      message = f"Error getting refresh token: {response.status_code} - {response.text}"
      raise ValueError(message)

def insta_api_get_access_token(client_id, client_secret, refresh_token):
    token_url = "https://api.ads.instacart.com/oauth/token"

    headers = {
        "Content-Type": "application/json"
    }

    data = {
        "client_id": client_id,
        "client_secret": client_secret,
        "redirect_uri": 'https://zvdataautomation.com/',
        "refresh_token": refresh_token,
        "grant_type": 'refresh_token',
    }

    response = requests.post(
        token_url,
        headers=headers,
        json=data,
    )

    try:
        access_token = response.json()['access_token']
        return access_token
    except requests.exceptions.RequestException as e:
        message = f"Error getting access token: {response.status_code} - {response.text}"
        raise ValueError(message)

def insta_api_get_product_report(start_date, end_date, file_path, access_token):
    # Request Report
    url = "https://api.ads.instacart.com/api/v2/reports/products"

    headers = {
        'Accept': 'application/json',
        'Content-Type': 'application/json',
        'Authorization': f'Bearer {access_token}',
    }

    payload = {
        'date_range': {
            'start_date': start_date,
            'end_date': end_date,
        },
        'segment': 'day',
    }

    response = requests.post(url, headers=headers, json=payload)
    try:
        response.raise_for_status() # Raise an exception for HTTP errors
        report_id = response.json()['data']['id']
        print(f'report_id: {report_id}')
    except requests.exceptions.RequestException as e:
        message = f"Error requesting Instacart product report: {response.status_code} - {response.text}"
        raise ValueError(message)

    # Check Report Status
    headers = {
        'Accept': 'application/json',
        'Authorization': f'Bearer {access_token}',
    }

    fileReady = _instacart_poll_report_status(report_id, access_token)

    if not fileReady:
        raise ValueError("Instacart report generation failed or timed out.")
    
    # Download the report
    url = f"https://api.ads.instacart.com/api/v3/reports/{report_id}/download"
    response = requests.get(url, headers=headers)
    fileName = os.path.join(file_path, f'instacart_report_{report_id}.csv')
    with open(fileName, 'wb') as f:
        f.write(response.content)
    print('Report Downloaded')
    return fileName

def insta_ads_report(filePath:str):
    instaDf = pd.read_csv(filePath)
    instaDf['date'] = pd.to_datetime(instaDf['date'])

    req_columns = [
        'date',
        'status',
        'campaign',
        'ad_group',
        'product',
        'product_size',
        'upc',
        'spend',
        'attributed_sales',
        'attributed_quantities',
        'roas',
        'impressions',
        'clicks',
        'ctr',
        'average_cpc',
        'ntb_attributed_sales',
        'percent_ntb_attributed_sales',
        'id',
        'campaign_uuid',
        'ad_group_uuid'
    ]

    missingColumns = set(req_columns) - set(instaDf.columns)
    newColumns = set(instaDf.columns) - set(req_columns)

    if missingColumns:
        message = (
        f"""
        missing columns: {', '.join(missingColumns)}
        """
        )
        raise ValueError(message)
    if newColumns:
        message = (
        f"""
        new columns: {', '.join(newColumns)}
        """
        )
        print(message)
    

    instaDf = instaDf[req_columns]

    schema = {
        'date' : 'datetime64[ns]',
        'status' : str,
        'campaign' : str,
        'ad_group' : str,
        'product' : str,
        'product_size' : str,
        'upc' : str,
        'spend' : float,
        'attributed_sales' : float,
        'attributed_quantities' : float,
        'roas' : float,
        'impressions' : float,
        'clicks' : float,
        'ctr' : float,
        'average_cpc' : float,
        'ntb_attributed_sales' : float,
        'percent_ntb_attributed_sales' : float,
        'id' : str,
        'campaign_uuid' : str,
        'ad_group_uuid' : str
    }

    instaDf = instaDf.astype(schema)
    return instaDf

def insta_api_get_marketshare(access_token, start_date, end_date):
    url = 'https://api.ads.instacart.com/api/v2/charts/tables'

    headers = {
        'Authorization': f'Bearer {access_token}',
        'Content-Type': 'application/json',
        'Accept': 'application/json',
    }

    payload = {
        "source": "sds",
        "query": {
            "series": {
                "columns": [
                    "sds.date",
                    "sds.brand_name",
                    "sds.category_name",
                    "sds.super_category_name",
                    "sds.department_name",
                    "sds.share",
                    "sds.brand_sales_usd",
                    "sds.brand_ad_spend_usd"
                ],
                "filters": [
                    {
                        "column": "sds.date",
                        "operation": "GTE",
                        "value": start_date
                    },
                    {
                        "column": "sds.date",
                        "operation": "LTE",
                        "value": end_date
                    },
                ],
                "order_bys": [
                    {
                        "column": "sds.date",
                        "direction": "DESC"
                    }
                ],
              "pagination": {
                  "page": 1,
                  "page_size": 500
              }
            }
        }
    }

    response = requests.post(url, headers=headers, json=payload)
    try:
        response.raise_for_status() # Raise an exception for HTTP errors
        columns_info = response.json()['data']['attributes']['header']['series']['columns']
        column_names = [col['name'].split('.')[-1] for col in columns_info]

        data_section = response.json()['data']['attributes']['data']['rows']
        shareDf = pd.DataFrame(data_section, columns=column_names)
    except requests.exceptions.RequestException as e:
        message = f"Error getting Instacart market share report: {response.status_code} - {response.text}"
        raise ValueError(message)

    req_columns = [
        'date',
        'brand_name',
        'category_name',
        'super_category_name',
        'department_name',
        'share',
        'brand_sales_usd',
        'brand_ad_spend_usd'
    ]

    missingColumns = set(req_columns) - set(shareDf.columns)
    newColumns = set(shareDf.columns) - set(req_columns)

    if missingColumns or newColumns:
        message = (
        f"""
        missing columns: {', '.join(missingColumns)}
        new columns: {', '.join(newColumns)}
        """
        )

        raise ValueError(message)

    shareDf = shareDf[req_columns]

    schema = {
        'date': 'datetime64[ns]',
        'brand_name': str,
        'category_name': str,
        'super_category_name': str,
        'department_name': str,
        'share': float,
        'brand_sales_usd': float,
        'brand_ad_spend_usd': float
        
    }
    
    shareDf = shareDf.astype(schema)
    return shareDf




import pandas as pd
from .gmail_automation import gchat_space

def krogerAdReport (report_file):
    df = pd.read_csv(report_file)
    df = df.rename(columns=lambda x:x.replace(' ', '_').replace('(','').replace(')','').replace('-','_').lower())
    df['daily_date'] = pd.to_datetime(df['daily_date'])
    df['purchased_upc'] = df['purchased_upc'].fillna(0).astype(int).astype(str)
    df['product_id'] = df['product_id'].fillna(0).astype(int).astype(str)
    colRep = df.columns[11:]
    df[colRep] = df[colRep].replace({'\$': '', ',': '', '%':''}, regex=True)

    req_columns = [
        'campaign_name',
        'advertiser_id',
        'advertiser_name',
        'ad_group_name',
        'daily_date',
        'product_id',
        'product_name',
        'purchased_upc',
        'purchased_product',
        'campaign_status',
        'ad_group_status',
        'impressions',
        'clicks',
        'cost',
        'cost_per_click_cpc',
        'click_through_rate_ctr',
        'clicked_transactions',
        'clicked_units',
        'clicked_revenue',
        'clicked_roas',
        'viewed_units',
        'viewed_transactions',
        'viewed_revenue',
        'viewed_roas',
        'cost_per_thousand_cpm'
    ]

    missingColumns = set(req_columns) - set(df.columns)
    newColumns = set(df.columns) - set(req_columns)

    if missingColumns:
        message = (f"""\nMissing columns: {', '.join(missingColumns)}""")
        raise ValueError(message)
    if newColumns:
        message = (f"""\nKroger Manula Report New columns:\n {', '.join(newColumns)}""")
        gchat_space(message, 'https://chat.googleapis.com/v1/spaces/AAAAq774XOA/messages?key=AIzaSyDdI0hCZtE6vySjMm-WEfRq3CPzqKqqsHI&token=2-eECUKfzs1-UsiN3-YG6IV-DzwrDSIoS-bKmqU2qZY')

    df = df[req_columns]

    schema = {
        'campaign_name': str,
        'advertiser_id': str,
        'advertiser_name': str,
        'ad_group_name': str,
        'daily_date': 'datetime64[ns]',
        'product_id': str,
        'product_name': str,
        'purchased_upc': str,
        'purchased_product': str,
        'campaign_status': str,
        'ad_group_status': str,
        'impressions': float,
        'clicks': float,
        'cost': float,
        'cost_per_click_cpc': float,
        'click_through_rate_ctr': float,
        'clicked_transactions': float,
        'clicked_units': float,
        'clicked_revenue': float,
        'clicked_roas': float,
        'viewed_units': float,
        'viewed_transactions': float,
        'viewed_revenue': float,
        'viewed_roas': float,
        'cost_per_thousand_cpm': float
    }
    df = df.astype(schema)

    return df




import asyncio
from playwright.async_api import async_playwright
import os
from datetime import datetime
import re

async def h10_asin_keyword_download(h10_email, h10_password, session_path, helium10_url, asins, start_date, end_date, folder_path, headless = True):
    print(f"[{datetime.now()}] 🔄 Starting Helium 10 session check...")

    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=headless)
        context = None

        # Try to load saved session
        if os.path.exists(session_path):
            try:
                context = await browser.new_context(storage_state=session_path)
                print("✅ Loaded existing session.")
            except Exception as e:
                print(f"⚠️ Error loading session: {e}")
                context = await browser.new_context()
        else:
            context = await browser.new_context()
            print("🆕 No session found. Starting fresh...")
                
        page = await context.new_page()
        await page.goto(helium10_url)

        try:
            # Check if logged in
            await page.wait_for_selector("#h10-style-container > div.sc-bztcrM.dETACx > header > div.sc-AqqLW.dMCPUg > nav > ul > ul > li.sc-jOltFJ.fjjKvL > a > svg", timeout=90000)
            print("✅ Already logged in.")
        except:
            print("🔐 Not logged in. Attempting login...")

            # await page.goto("https://members.helium10.com/")
            await page.wait_for_selector("#loginform-email")
            await page.fill("#loginform-email", h10_email)
            await page.fill("#loginform-password", h10_password)
            await page.click("#login-form > button")

            # Wait for dashboard/redirect
            try:
                await page.wait_for_selector("#h10-style-container > div.sc-bztcrM.dETACx > header > div.sc-AqqLW.dMCPUg > nav > ul > ul > li.sc-jOltFJ.fjjKvL > a > svg", timeout=90000)
                print("✅ Login successful.")
                await context.storage_state(path=session_path)
                print("💾 Session saved to session.json")
            except:
                raise ValueError("❌ Login to H10 failed.")

        print(f"[{datetime.now()}] ✅ Session ready.")
        
        # Wait for the search bar
        await page.reload()
        await asyncio.sleep(3)
        await page.reload()
        await asyncio.sleep(3)
        await page.reload()
        await page.wait_for_selector("#table-search-input", timeout=180000)
        
        for asin in asins:
            try:
                try:
                    await page.locator("div").filter(has_text=re.compile(r"^1 ProductCustomizeExport Data\.\.\.Add ProductsOpen Walkthrough$")).locator("#table-search-input").click()
                    await page.locator("div").filter(has_text=re.compile(r"^1 ProductCustomizeExport Data\.\.\.Add ProductsOpen Walkthrough$")).locator("#table-search-input").press("ControlOrMeta+a")
                    await page.locator("div").filter(has_text=re.compile(r"^1 ProductCustomizeExport Data\.\.\.Add ProductsOpen Walkthrough$")).locator("#table-search-input").fill(asin)
                    print(f"Searching for ASIN: {asin}")
                    await asyncio.sleep(1)
                    await page.locator("div").filter(has_text=re.compile(r"^1 ProductCustomizeExport Data\.\.\.Add ProductsOpen Walkthrough$")).locator("#table-search-input").press("Enter")
                except:
                    await page.locator("#table-search-input").click()
                    await page.locator("#table-search-input").fill(asin)
                    print(f"Searching for ASIN: {asin}")
                    await asyncio.sleep(1)
                    await page.locator("#table-search-input").press("Enter")

                # Click the Keywords Dropdown button
                kw_drop_button = page.get_by_test_id("table-cell-actions").get_by_role("button").nth(1)
                await kw_drop_button.wait_for(state='visible',timeout=15000)
                await kw_drop_button.click()

                # Click the "History" button
                export_button = page.locator("#keyword-tracked-keywords_wrapper").get_by_role("button", name="Export Data...")
                await export_button.wait_for(state='visible', timeout=90000)
                await export_button.click()

                # Wait 1 second and click the first dropdown option
                await asyncio.sleep(1)
                await page.get_by_text("Historical Data").click()

                # Wait for popup to appear and fill date
                date_textbox = page.get_by_role("textbox", name="Select a date")
                await date_textbox.wait_for(state='visible', timeout=15000)
                await date_textbox.fill(f"{start_date} - {end_date}")
                await asyncio.sleep(1)
                print("Date filled in successfully.")

                # Click outside the date box and download
                await page.get_by_role("heading", name="Export Historical Data Select").click()

                # Wait for popup to close (indicates download triggered)
                async with page.expect_download(timeout=90000) as download_info:
                    await page.get_by_role("button", name="Export", exact=True).click()
                download = await download_info.value
                
                await download.save_as(os.path.join(folder_path, f"{asin}_report.csv"))
                print(f"{asin}_report.csv Downloaded successfully.")

            except Exception as e:
                print(f"Error processing ASIN {asin}: {e}")
                continue

            await asyncio.sleep(3)
        
        await browser.close()

        file_count = len([f for f in os.listdir(folder_path) if f.endswith('.csv')])
        done_message = (f"✅ Total of {file_count} ASINs processed successfully.")
        return done_message